#! /usr/bin/python

import os, sys

def launch(args):
    cairo_home = ''
    cairo_version='0.1' # todo: dangerous default; see below
    try: from config import cairo_home, cairo_version
    except: pass

    osname = os.name
    if osname != 'nt':
        report_error("""\
    ERROR: this script not supported for %s.
    You will need to modify this script for it to work with
    your operating system.""" % osname)

    if args:
        for i in range(len(args)):
            if args[i].startswith('file:'):
                args[i] = args[i].replace('CAIRO_HOME', cairo_home)
    else:
        report_error("""ERROR: improper call to launch script
    launch.bat should not be executed directly, please see README for
    proper application launching instructions.""")

    if not cairo_home:
        if osname == 'nt':
            cairo_home = os.path.join(os.getcwd(), '..')
            # todo: get actual cairo_version from os.listdir('..')
        if not cairo_home and cairo_version:
            report_error("""ERROR: CAIRO_HOME not found in your environment.
    Please set the CAIRO_HOME variable in your environment to match the
    location of the Cairo installation.""")

    cairo_jar = os.path.join(cairo_home, 'cairo-%s.jar' % cairo_version)

    if not os.path.exists(cairo_jar):
        report_error("""ERROR: CAIRO_HOME is set to an invalid directory.
    CAIRO_HOME = %s
    %s not found!
    Please set the CAIRO_HOME variable in your environment to match the
    location of the Cairo installation.""" % (cairo_home, cairo_jar))

    if 'JAVA_HOME' not in os.environ:
        report_error("""ERROR: JAVA_HOME not found in your environment.
    Please set the JAVA_HOME variable in your environment to match the
    location of your Java installation.""")
    java_home = os.environ['JAVA_HOME']

    if not os.path.exists(os.path.join(java_home, 'bin', 'java.exe')):
        report_error("""ERROR: JAVA_HOME is set to an invalid directory.
    JAVA_HOME = %s
    Please set the JAVA_HOME variable in your environment to match the
    location of your Java installation.""" % java_home)

    if not ((os.path.exists(os.path.join(java_home,
                                         'jre', 'lib', 'ext', 'jmf.jar'))) \
            or (os.path.exists(os.path.join(java_home,
                                             'lib', 'ext', 'jmf.jar')))):
        report_error("""ERROR: Java Media Framework (JMF) is not installed.
    Please download and install JMF from Sun Java web site:
    http://java.sun.com/products/java-media/jmf/.""")

    if not ((os.path.exists(os.path.join(java_home,
                                         'jre', 'lib', 'ext', 'jsapi.jar'))) \
            or (os.path.exists(os.path.join(java_home,
                                             'lib', 'ext', 'jsapi.jar')))    \
            or (os.path.exists(os.path.join(cairo_home,
                                             'lib', 'jsapi.jar')))):
        report_error("""ERROR: Java Speech API (JSAPI) is not installed.
    Please run jsapi.exe or jsapi.sh and place the extracted
    jsapi.jar in %s\jre\lib\ext
    The install file can be downloaded from here:
    http://www.speechforge.org/downloads/jsapi.""" % java_home)

    lib_dir = os.path.join(cairo_home, 'lib')
    cairo_lib_jars = ';'.join([os.path.join(lib_dir, fn)
                               for fn in os.listdir(lib_dir)
                               if os.path.splitext(fn)[1] == '.jar'])

    classpath = '%s;%s;%s;' % (cairo_jar, cairo_lib_jars,
                              os.path.join(cairo_home, 'config'))
    os.environ['CLASSPATH'] = '%s%s' % (os.environ['CLASSPATH'], classpath)

##     # works OK for launch scripts but not for demo scripts
##     java_exe = os.path.join(java_home, 'bin', 'java')
##     cmd = '"%s" -Xmx200m -Dlog4j.configuration=log4j.xml %s' \
##           % (java_exe, ' '.join(args))
    cmd = 'java -Xmx200m -Dlog4j.configuration=log4j.xml %s' \
          % (' '.join(args))
    print cmd
    os.system(cmd)

def launch_resource(resType, resName):
    klass = 'org.speechforge.cairo.server.resource.%sResource' \
            % resType.capitalize()
    cairo_config = 'file:CAIRO_HOME/config/cairo-config.xml' # works for now
    args = [klass, cairo_config, resName]
    launch(args)

def report_error(message):
    print message
    print_usage()
    sys.exit(1)

def print_usage():
    print """
* launch.py:  Cairo MRCP Server launch script

** Usage

    launch.py resourceClass (config) (resourceName)

** Examples

    launch.py org.speechforge.cairo.server.resource.ResourceServerImpl
    launch.py org.speechforge.cairo.server.resource.ReceiverResource file:../config/cairo-config.xml receiver1

For obvious reasons, it is best to use the helper scripts:

rserver.py
receiver1.py
transmitter1.py
"""

if __name__ == '__main__':
    args = sys.argv
    if args[0].lower().startswith('python'): args.pop(0)
    args.pop(0) # removes script name
    if not args:
        report_error('ERROR: no arguments!')
    launch(args)
